struct mat;
struct vec;

inline float dot(const vec &a, const vec &b);

struct vec
{
   float x,y,z;

   ::vec(float px, float py, float pz):x(px),y(py),z(pz) { }
   ::vec() { x=y=z=0; }

   vec operator*(float a) const {
      return vec(x*a,y*a,z*a);
   }

   vec operator+(const vec &a) const {
      return vec(x+a.x,y+a.y,z+a.z);
   }
   vec operator-(const vec &a) const {
      return vec(x-a.x,y-a.y,z-a.z);
   }

   vec& operator+=(const vec &a) {
      x += a.x; y += a.y; z += a.z;
      return *this;
   }

   vec& operator*=(float a) {
      x *= a; y *= a; z *= a;
      return *this;
   }

   bool operator==(const vec &a) const {
      return x==a.x && y == a.y && z == a.z;
   }

   float normalize() {
      float len2 = dot(*this,*this);
      float len = sqrt(len2);
      *this *= 1/len;
      return len;
   }

   vec normal() const {
      float len2 = dot(*this,*this);
      float len = sqrt(len2);
      float len_i = 1.0f / len;
      return vec(x*len_i, y*len_i, z*len_i);
   }

   float length() const { return sqrt(dot(*this,*this)); }
   float len2() const { return dot(*this,*this); }

   vec operator*(const mat &m) const;

   float& operator[](int n) { return (&x)[n]; }
   float operator[](int n) const { return (&x)[n]; }

   vec mul(const vec &b) const {
      return vec(x*b.x, y*b.y, z*b.z);
   }
};

inline vec cross(const vec &a, const vec &b) {
   return vec(a.y*b.z - a.z*b.y,
              a.z*b.x - a.x*b.z,
              a.x*b.y - a.y*b.z);
}

inline float dist(const vec &a, const vec &b) {
   return (b-a).length();
}

inline vec lerp(const vec &a, const vec &b, float t) {
   return a + (b-a)*t;
}

inline vec operator*(float a, const vec &b) {
   return vec(a*b.x, a*b.y, a*b.z);
}

inline float dot(const vec &a, const vec &b) {
   return a.x*b.x + a.y*b.y + a.z*b.z;
}

struct mat
{
   vec v[3];

   // mat * vec
   vec operator*(const vec &a) const    {
      return vec(dot(v[0], a),
                 dot(v[1], a),
                 dot(v[2], a));
   }

   mat transpose() const {
      mat o;
      for (int j=0; j < 3; ++j)
      for (int i=0; i < 3; ++i)
         o.v[j][i] = this->v[i][j];
      return o;
   }

   mat operator*(const mat &b) const {
      const mat &a = *this;
      mat o;
      for (int j=0; j < 3; ++j)
      for (int i=0; i < 3; ++i)
         o.v[j][i] = a.v[j][0] * b.v[0][i]
                   + a.v[j][1] * b.v[1][i]
                   + a.v[j][2] * b.v[2][i];
      return o;
   }

   void identity() {
      memset(this, 0, sizeof(vec)*3);
      v[0][0] = v[1][1] = v[2][2] = 1;
   }
};

extern mat rotationMatrix(float ang, const vec &axis);

// multiply by vector on left == multiply by transpose
inline vec vec::operator*(const mat &m) const
{
   return m.transpose() * *this;
}

extern void glmat(float out[16], const mat &ori, const vec &loc);
